@@ -5,7 +5,7 @@
 #' 
 #' The idealized trajectory is defined as the straight line connecting the start
 #' and end point of the actual trajectory (e.g., Freeman & Ambady, 2010). The 
-#' deviation for each position is calclated as the perpendicular deviation of 
+#' deviation for each position is calculated as the perpendicular deviation of 
 #' the actual trajectory from the idealized trajectory.
 #' 
 #' If a deviation occurs above the direct path, this is denoted by a positive 
@@ -33,8 +33,8 @@
 #'   of the idealized trajectory (by default called \code{xpos_ideal} and 
 #'   \code{ypos_ideal}) and the perpendicular deviations of the actual 
 #'   trajectory from the idealized trajectory (by default called 
-#'   \code{dev_ideal}) have been added as additional columns to the trajectory
-#'   array. If the trajectory array was provided directly as \code{data}, only
+#'   \code{dev_ideal}) have been added as additional variables to the trajectory
+#'   array. If the trajectory array was provided directly as \code{data}, only 
 #'   the trajectory array will be returned.
 #'
 #' @references Freeman, J. B., & Ambady, N. (2010). MouseTracker: Software for
@@ -52,9 +52,13 @@
 #' # Calculate deviations from idealized trajectory with
 #' # constant start and end points across trials
 #' mt_example <- mt_deviations(mt_example,
-#'   start_ideal=c(xpos=0,ypos=0), end_ideal=c(xpos=-665,ypos=974))
+#'   start_ideal=c(0,0), end_ideal=c(-665,974))
+#' 
+#' @author
+#' Pascal J. Kieslich (\email{kieslich@@psychologie.uni-mannheim.de})
+#' 
+#' Felix Henninger
 #' 
-#' @describeIn mt_deviations Calculate deviations from idealized trajectory
 #' @export
 mt_deviations <- function(
   data,
@@ -62,14 +66,8 @@ mt_deviations <- function(
   dimensions=c("xpos","ypos"),
   start_ideal=NULL,end_ideal=NULL,
   prefix="",
-  verbose=FALSE,show_progress=NULL) {
+  verbose=FALSE) {
   
-  if(is.null(show_progress)==FALSE){
-    warning("The argument show_progress is deprecated. ",
-            "Please use verbose instead.",
-            call. = FALSE)
-    verbose <- show_progress
-  }
   
   if(length(dimensions)!=2){
     stop("For dimensions, exactly two trajectory dimensions have to be specified.")
@@ -85,13 +83,13 @@ mt_deviations <- function(
     variables=c(points_ideal,dev_ideal))
   
   # Calculate number of logs
-  nlogs <- rowSums(!is.na(deviations[,dimensions[[1]],,drop=FALSE]))
+  nlogs <- mt_count(deviations,dimensions=dimensions[[1]])
   
   
   # Calculate deviations
   for (i in 1:nrow(deviations)){
     
-    current_points <- deviations[i, dimensions, 1:nlogs[i]]
+    current_points <- deviations[i, 1:nlogs[i], dimensions]
     
     # Determine straight line (idealized trajectory)
     current_points_ideal <- points_on_ideal(
@@ -99,21 +97,21 @@ mt_deviations <- function(
     
     # Calculate distance of each point on the curve from straight line
     # (cf. Pythagoras, some time ago)
-    current_dev_ideal <- sqrt(colSums((current_points_ideal-current_points)^2))
+    current_dev_ideal <- sqrt(rowSums((current_points_ideal-current_points)^2))
     
     # Flip signs for points below the idealized straight line
     flip_dimension <- 2
-    to_flip_dev_ideal <- current_points_ideal[flip_dimension,]>current_points[flip_dimension,]
+    to_flip_dev_ideal <- current_points_ideal[,flip_dimension]>current_points[,flip_dimension]
     current_dev_ideal[to_flip_dev_ideal] <- -current_dev_ideal[to_flip_dev_ideal]
     
     # If last point of actual trajectory is below the first point, flip all points
-    if(current_points[flip_dimension,1]>current_points[flip_dimension,nlogs[i]]){
+    if(current_points[1,flip_dimension]>current_points[nlogs[i],flip_dimension]){
       current_dev_ideal <- -(current_dev_ideal)
     }
     
     # Add idealized positions and deviations to array
-    deviations[i,points_ideal,1:nlogs[i]] <- current_points_ideal
-    deviations[i,dev_ideal,1:nlogs[i]] <- current_dev_ideal 
+    deviations[i,1:nlogs[i],points_ideal] <- current_points_ideal
+    deviations[i,1:nlogs[i],dev_ideal] <- current_dev_ideal 
     
     if (verbose){
       if (i %% 100 == 0) message(paste(i,"trials finished"))
@@ -127,28 +125,3 @@ mt_deviations <- function(
   return(create_results(data=data, results=deviations, use=use, save_as=save_as))
   
 }
-
-
-#' @describeIn mt_deviations Deprecated
-#' @export
-mt_calculate_deviations <- function(
-  data,
-  use="trajectories", save_as=use,
-  dimensions=c("xpos","ypos"),
-  start_ideal=NULL,end_ideal=NULL,
-  prefix="",
-  verbose=FALSE,show_progress=NULL) {
-  
-  .Deprecated("mt_deviations")
-  
-  return(
-    mt_deviations(
-      data=data,use=use,save_as=save_as,
-      dimensions=dimensions,
-      start_ideal=start_ideal,end_ideal=end_ideal,
-      prefix=prefix,
-      verbose=verbose,show_progress=show_progress
-    )
-  )
-  
-}
\ No newline at end of file
