@@ -1,16 +1,14 @@
-#' Import mouse-tracking data recorded using the mousetrap plug-ins in
-#' OpenSesame.
+#' Import mouse-tracking data recorded using the mousetrap plug-ins in OpenSesame.
 #'
-#' \code{mt_import_mousetrap} accepts a data.frame of (merged) raw data from a
-#' mouse-tracking experiment implemented in OpenSesame using the
-#' \href{https://github.com/pascalkieslich/mousetrap-os}{mousetrap plugin}. From
-#' this data.frame, \code{mt_import_mousetrap} creates a mousetrap data object
-#' containing the trajectories and additional data for further processing within
-#' the mousetrap package. Specifically, it returns a list that includes the
-#' trajectory data as an array (called \code{trajectories}), and all other data
-#' as a data.frame (called \code{data}). This data structure can then be passed
-#' on to other functions within this package (see \link{mousetrap} for an
-#' overview).
+#' \code{mt_import_mousetrap} accepts a data.frame of (merged) raw data from a 
+#' mouse-tracking experiment implemented in OpenSesame using the mousetrap 
+#' plugin (Kieslich & Henninger, 2017). From this data.frame,
+#' \code{mt_import_mousetrap} creates a mousetrap data object containing the
+#' trajectories and additional data for further processing within the mousetrap
+#' package. Specifically, it returns a list that includes the trajectory data as
+#' an array (called \code{trajectories}), and all other data as a data.frame
+#' (called \code{data}). This data structure can then be passed on to other
+#' functions within this package (see \link{mousetrap} for an overview).
 #'
 #' When working with mouse-tracking data that were recorded using the mousetrap
 #' plug-ins for OpenSesame, usually only the \code{raw_data} need to be
@@ -66,15 +64,18 @@
 #'   is recommended for all following analyses in mousetrap.
 #' @param verbose logical indicating whether function should report its
 #'   progress.
-#' @param show_progress Deprecated. Please use \code{verbose} instead.
 #'
 #' @return A mousetrap data object (see \link{mt_example}).
 #'
 #'   If mouse-tracking data were recorded using the mousetrap plug-ins for
 #'   OpenSesame, the unit of the timestamps is milliseconds.
 #'
-#' @references Mousetrap plug-ins for OpenSesame
-#'   (\url{https://github.com/pascalkieslich/mousetrap-os})
+#' @references Kieslich, P. J., & Henninger, F. (2016). Mousetrap: 
+#'   Mouse-tracking plugins for OpenSesame (Version 1.2.1). doi: 
+#'   \url{https://doi.org/10.5281/zenodo.163404}
+#'   
+#'   Kieslich, P. J., & Henninger, F. (2017). Mousetrap: An integrated, 
+#'   open-source mouse-tracking package. Manuscript submitted for publication.
 #'
 #' @seealso \link[readbulk]{read_opensesame} from the \code{readbulk} library
 #'   for reading and combining raw data files that were collected with
@@ -86,6 +87,11 @@
 #' @examples
 #' mt_data <- mt_import_mousetrap(mt_example_raw)
 #'
+#' @author
+#' Pascal J. Kieslich (\email{kieslich@@psychologie.uni-mannheim.de})
+#' 
+#' Felix Henninger
+#' 
 #' @export
 mt_import_mousetrap <- function(raw_data,
   xpos_label="xpos", ypos_label="ypos",
@@ -93,17 +99,8 @@ mt_import_mousetrap <- function(raw_data,
   mt_id_label=NULL,
   split=",", duplicates="remove_first",
   reset_timestamps=TRUE,
-  verbose=FALSE, show_progress=NULL) {
-
-  if(is.null(show_progress) == FALSE) {
-    warning(
-      "The argument show_progress is deprecated. ",
-      "Please use verbose instead.",
-      call.=FALSE
-    )
-    verbose <- show_progress
-  }
-
+  verbose=FALSE) {
+  
   # Set labels
   timestamps <- "timestamps"
   xpos <- "xpos"
@@ -235,8 +232,8 @@ mt_import_mousetrap <- function(raw_data,
 
   # Create array with raw MT data
   trajectories <- array(
-    dim=c(nrow(raw_data), 3, max_logs),
-    dimnames=list(raw_data[,"mt_id"], c(timestamps,xpos,ypos), NULL)
+    dim=c(nrow(raw_data), max_logs, 3),
+    dimnames=list(raw_data[,"mt_id"], NULL, c(timestamps,xpos,ypos))
   )
 
   for (i in 1:dim(trajectories)[1]) {
@@ -246,21 +243,21 @@ mt_import_mousetrap <- function(raw_data,
         mt_l <- columns[mt_labels[j]]
 
         if(length(data_list[i,][[mt_l]]) > 0){ ## only extract if there is data
-          trajectories[i, j, 1:length(data_list[i,][[mt_l]])] <- data_list[i,][[mt_l]]
+          trajectories[i, 1:length(data_list[i,][[mt_l]]), j] <- data_list[i,][[mt_l]]
         }
 
       }
     # Special case (only one trajectory or equal number of logs)
     } else {
-      trajectories[i,timestamps,] <- data_list[,i,columns[timestamps_label]]
-      trajectories[i,xpos,] <- data_list[,i,columns[xpos_label]]
-      trajectories[i,ypos,] <- data_list[,i,columns[ypos_label]]
+      trajectories[i,,timestamps] <- data_list[,i,columns[timestamps_label]]
+      trajectories[i,,xpos] <- data_list[,i,columns[xpos_label]]
+      trajectories[i,,ypos] <- data_list[,i,columns[ypos_label]]
     }
 
     # Check timestamps
 
     # Extract timestamps
-    current_timestamps <- trajectories[i,timestamps,]
+    current_timestamps <- trajectories[i,,timestamps]
     current_timestamps <- current_timestamps[1:sum(!is.na(current_timestamps))]
 
     # Check that timestamps are monotonically increasing
@@ -275,15 +272,15 @@ mt_import_mousetrap <- function(raw_data,
       if (duplicates %in% c("remove_first", "remove_last")) {
 
         if (anyDuplicated(current_timestamps) > 0) {
-          current_xpos <- trajectories[i, xpos, 1:length(current_timestamps)]
-          current_ypos <- trajectories[i, ypos, 1:length(current_timestamps)]
+          current_xpos <- trajectories[i, 1:length(current_timestamps), xpos]
+          current_ypos <- trajectories[i, 1:length(current_timestamps), ypos]
           trajectories[i,,] <- NA
 
           keep <- !duplicated(current_timestamps, fromLast = duplicates=="remove_first")
           current_timestamps <- current_timestamps[keep]
-          trajectories[i,timestamps,1:length(current_timestamps)] <- current_timestamps
-          trajectories[i,xpos,1:length(current_timestamps)] <- current_xpos[keep]
-          trajectories[i,ypos,1:length(current_timestamps)] <- current_ypos[keep]
+          trajectories[i,1:length(current_timestamps),timestamps] <- current_timestamps
+          trajectories[i,1:length(current_timestamps),xpos] <- current_xpos[keep]
+          trajectories[i,1:length(current_timestamps),ypos] <- current_ypos[keep]
         }
 
       } else {
@@ -305,8 +302,8 @@ mt_import_mousetrap <- function(raw_data,
   }
 
   # Check if there are trials with no logs
-  if (any(is.na(trajectories[,,1]))) {
-    na_trials <- rowSums(is.na(trajectories[,,1,drop=FALSE]))
+  if (any(is.na(trajectories[,1,]))) {
+    na_trials <- rowSums(is.na(trajectories[,1,,drop=FALSE]))
     warning(paste(
       "The following trials do not contain any logging data for at least one variable:",
       paste(names(na_trials[na_trials>0]), collapse=", ")
@@ -314,7 +311,7 @@ mt_import_mousetrap <- function(raw_data,
   }
 
   # Check for each trial, if the number of logs is the same for every variale
-  nlogs <- apply(trajectories, c(1,2), function(x) {sum(!is.na(x))})
+  nlogs <- apply(trajectories, c(1,3), function(x) {sum(!is.na(x))})
   nlogs_sd <- apply(nlogs, 1, stats::sd)
   if (any(nlogs_sd > 0)) {
     warning(paste(
@@ -326,7 +323,7 @@ mt_import_mousetrap <- function(raw_data,
 
   # Subtract first timestamp for each trial
   if (reset_timestamps) {
-    trajectories[,timestamps,] <- trajectories[,timestamps,] - trajectories[,timestamps,1]
+    trajectories[,,timestamps] <- trajectories[,,timestamps] - trajectories[,1,timestamps]
   }
 
   # Drop raw data columns
@@ -336,38 +333,37 @@ mt_import_mousetrap <- function(raw_data,
   } else {
     raw_data <- raw_data[, !colnames(raw_data) %in% columns]
   }
+  
+  result <- c(list("data"=raw_data, "trajectories"=trajectories))
+  class(result) <- "mousetrap"
 
-  return(c(list("data"=raw_data, "trajectories"=trajectories)))
+  return(result)
 }
 
 
 #' Import mouse-tracking data saved in wide format.
 #'
-#' \code{mt_import_wide} receives a data.frame where mouse-tracking data are
+#' \code{mt_import_wide} receives a data.frame where mouse-tracking data are 
 #' stored in wide format, i.e., where one row contains the data of one trial and
-#' every recorded mouse position and variable is saved in a separate variable
-#' (e.g., X_1, X_2, ..., Y_1, Y_2, ...). This is, e.g., the case when exporting
-#' trajectories from MouseTracker (Freeman & Ambady, 2010). From this
-#' data.frame, \code{mt_import_wide} creates a mousetrap data object containing
-#' the trajectories and additional data for further processing within the
-#' mousetrap package. Specifically, it returns a list that includes the
-#' trajectory data as an array (called \code{trajectories}), and all other data
-#' as a data.frame (called \code{data}). This data structure can then be passed
-#' on to other functions within this package (see \link{mousetrap} for an
-#' overview).
+#' every recorded mouse position and variable is saved in a separate variable 
+#' (e.g., X_1, X_2, ..., Y_1, Y_2, ...). This is, e.g., the case when collecting
+#' data using MouseTracker (Freeman & Ambady, 2010). From this data.frame,
+#' \code{mt_import_wide} creates a mousetrap data object containing the
+#' trajectories and additional data for further processing within the mousetrap
+#' package. Specifically, it returns a list that includes the trajectory data as
+#' an array (called \code{trajectories}), and all other data as a data.frame
+#' (called \code{data}). This data structure can then be passed on to other
+#' functions within this package (see \link{mousetrap} for an overview).
 #'
-#' \code{mt_import_wide} is designed to import mouse-tracking data saved in a
-#' wide format. The defaults are set so that usually only the \code{raw_data}
-#' and \code{pos_ids} need to be provided when importing trajectory data that
-#' stem from a "time normalized analysis" in MouseTracker (Freeman & Ambady,
-#' 2010).
+#' \code{mt_import_wide} is designed to import mouse-tracking data saved in a 
+#' wide format. The defaults are set so that usually only the \code{raw_data} 
+#' need to be provided when data have been collecting using MouseTracker
+#' (Freeman & Ambady, 2010) and have been read into R using \link{read_mt}.
 #'
-#' If no \code{pos_ids} are provided, column labels for the respective variable
-#' (e.g., x-positions) are extracted using \code{\link{grep}} returning all
-#' variables that start with the respective character string (e.g., "X_" if
-#' \code{xpos_label="X"} and \code{pos_sep="_"}). This is, e.g., useful when
-#' importing trajectory data that stem from a "raw time analysis" in
-#' MouseTracker (Freeman & Ambady, 2010).
+#' If no \code{pos_ids} are provided, column labels for the respective variable 
+#' (e.g., x-positions) are extracted using \code{\link{grep}} returning all 
+#' variables that start with the respective character string (e.g., "X_" if 
+#' \code{xpos_label="X"} and \code{pos_sep="_"}).
 #'
 #' If no timestamps are found in the data, \code{mt_import_wide} automatically
 #' creates a timestamps variable with increasing integers (starting with 0)
@@ -400,9 +396,10 @@ mt_import_mousetrap <- function(raw_data,
 #' studying real-time mental processing using a computer mouse-tracking method.
 #' \emph{Behavior Research Methods, 42}(1), 226-241.
 #'
-#' @seealso \link{read_mousetracker} for reading data into R that were exported
-#' from MouseTracker (Freeman & Ambady, 2010).
-#'
+#' @seealso \link{read_mt} for reading raw data that was collected using
+#'   MouseTracker (Freeman & Ambady, 2010) and stored as a file in the ".mt"
+#'   format.
+#' 
 #' \link{mt_import_mousetrap} and \link{mt_import_long} for importing
 #' mouse-tracking data in other formats.
 #'
@@ -413,20 +410,14 @@ mt_import_mousetrap <- function(raw_data,
 #'
 #' # Import the data using mt_import_wide
 #' mt_data <- mt_import_wide(mt_data_wide,
-#'   xpos_label="xpos", ypos_label="ypos", timestamps="timestamps")
-#'
-#'
-#' \dontrun{
-#'
-#' # Data from "time normalized analysis" in MouseTracker
-#' raw_data <- read_mousetracker("tn_data_exported.csv")
-#' data <- mt_import_wide(raw_data, pos_ids=1:101)
-#'
-#' # Data from "raw time analysis" in MouseTracker
-#' raw_data <- read_mousetracker("raw_data_exported.csv", last_lines_to_rm=2*8)
-#' data <- mt_import_wide(raw_data)
-#'
-#' }
+#'   xpos_label="xpos", ypos_label="ypos",
+#'   timestamps_label="timestamps")
+#' 
+#' @author
+#' Pascal J. Kieslich (\email{kieslich@@psychologie.uni-mannheim.de})
+#' 
+#' Felix Henninger
+#' 
 #' @export
 mt_import_wide <- function(raw_data,
   xpos_label="X", ypos_label="Y", zpos_label=NULL,
@@ -558,11 +549,11 @@ mt_import_wide <- function(raw_data,
   max_logs <- max(sapply(mt_columns,length))
 
   trajectories <- array(
-    dim=c(nrow(raw_data), length(mt_labels), max_logs),
-    dimnames=list(raw_data[,"mt_id"], names(mt_labels), NULL))
+    dim=c(nrow(raw_data), max_logs, length(mt_labels)),
+    dimnames=list(raw_data[,"mt_id"], NULL, names(mt_labels)))
 
   for (mt_var in names(mt_labels)) {
-    trajectories[,mt_var,] <- as.matrix(raw_data[,mt_columns[[mt_var]]])
+    trajectories[,,mt_var] <- as.matrix(raw_data[,mt_columns[[mt_var]]])
     raw_data <- raw_data[, !colnames(raw_data) %in% mt_columns[[mt_var]], drop=FALSE]
   }
 
@@ -575,12 +566,12 @@ mt_import_wide <- function(raw_data,
       )
     }
     timestamps_matrix <- matrix(
-      0:(dim(trajectories)[3]-1),
-      nrow=nrow(trajectories), ncol=dim(trajectories)[3],
+      0:(dim(trajectories)[2]-1),
+      nrow=nrow(trajectories), ncol=dim(trajectories)[2],
       byrow=TRUE
     )
     # Add NAs for timestamps (corresponding to NAs for first dimension)
-    timestamps_matrix[is.na(trajectories[,1,])] <- NA
+    timestamps_matrix[is.na(trajectories[,,1])] <- NA
 
     # Add timestamps to trajectories
     trajectories <- mt_add_variables(trajectories, variables=list(timestamps=timestamps_matrix))
@@ -590,13 +581,15 @@ mt_import_wide <- function(raw_data,
   # if real timestamps are provided and option was selected
   } else {
     if (reset_timestamps) {
-      trajectories[,timestamps,] <- trajectories[,timestamps,] - trajectories[,timestamps,1]
+      trajectories[,,timestamps] <- trajectories[,,timestamps] - trajectories[,1,timestamps]
     }
   }
-
-
-  return(c(list("data"=raw_data, "trajectories"=trajectories)))
-
+  
+  result <- c(list("data"=raw_data, "trajectories"=trajectories))
+  class(result) <- "mousetrap"
+  
+  return(result)
+  
 }
 
 
@@ -606,15 +599,16 @@ mt_import_wide <- function(raw_data,
 #' stored in long format, i.e., where one row contains the logging data
 #' (timestamp, x- and y-position etc.) at one specific point in the trial. This
 #' is, for example, the case when exporting the trajectory data from the
-#' mousetrap package using \link{mt_reshape}. From this data.frame,
+#' mousetrap package using \link{mt_export_long}. From this data.frame,
 #' \code{mt_import_long} creates a mousetrap data object containing the
 #' trajectories and additional data for further processing within the mousetrap
 #' package. Specifically, it returns a list that includes the trajectory data as
 #' an array (called \code{trajectories}), and all other data as a data.frame
 #' (called \code{data}). This data structure can then be passed on to other
-#' functions within this package (see \link{mousetrap} for an overview). The
-#' defaults are set so that no adjustments have to be made when importing a
-#' data.frame that was created using \link{mt_reshape}.
+#' functions within this package (see \link{mousetrap} for an overview).
+#' 
+#' The default arguments are set so that no adjustments have to be made when
+#' importing a data.frame that was created using \link{mt_export_long}.
 #'
 #' The coordinates are ordered according to the values in the column provided in
 #' the \code{mt_seq_label} parameter (\code{mt_seq} by default). If the
@@ -670,6 +664,12 @@ mt_import_wide <- function(raw_data,
 #' mt_data <- mt_import_long(exp_data,
 #'   add_labels= c("angle", "velocity"))
 #' }
+#' 
+#' @author
+#' Pascal J. Kieslich (\email{kieslich@@psychologie.uni-mannheim.de})
+#' 
+#' Felix Henninger
+#' 
 #' @export
 mt_import_long <- function(raw_data,
   xpos_label="xpos", ypos_label="ypos", zpos_label=NULL,
@@ -753,14 +753,14 @@ mt_import_long <- function(raw_data,
   n_max <- max(n_logs$n)
 
   trajectories <- array(
-    dim = c(nrow(n_logs),length(mt_include), n_max),
-    dimnames = list(n_logs$mt_id, mt_include, NULL))
+    dim = c(nrow(n_logs),n_max, length(mt_include)),
+    dimnames = list(n_logs$mt_id, NULL, mt_include))
 
   for (var in mt_include) {
     reshaped_data <- raw_data %>%
       dplyr::select_(.dots=c("mt_id", "mt_seq", var)) %>%
       tidyr::spread_("mt_seq", var)
-    trajectories[,var,] <- as.matrix(reshaped_data[,-1])
+    trajectories[,,var] <- as.matrix(reshaped_data[,-1])
     }
 
   # If no timestamps are found in the data, create timestamps
@@ -773,12 +773,12 @@ mt_import_long <- function(raw_data,
     }
 
     timestamps_matrix <- matrix(
-      0:(dim(trajectories)[3]-1),
-      nrow=nrow(trajectories), ncol=dim(trajectories)[3],
+      0:(dim(trajectories)[2]-1),
+      nrow=nrow(trajectories), ncol=dim(trajectories)[2],
       byrow=TRUE
     )
     # Add NAs for timestamps (corresponding to NAs for first dimension)
-    timestamps_matrix[is.na(trajectories[,1,])] <- NA
+    timestamps_matrix[is.na(trajectories[,,1])] <- NA
 
     # Add timestamps to trajectories
     trajectories <- mt_add_variables(trajectories, variables=list(timestamps=timestamps_matrix))
@@ -787,7 +787,7 @@ mt_import_long <- function(raw_data,
   # if real timestamps are provided and option was selected
   } else {
     if (reset_timestamps) {
-      trajectories[,timestamps,] <- trajectories[,timestamps,] - trajectories[,timestamps,1]
+      trajectories[,,timestamps] <- trajectories[,,timestamps] - trajectories[,1,timestamps]
     }
   }
 
@@ -813,6 +813,9 @@ mt_import_long <- function(raw_data,
     # Ensure order of raw_data
     raw_data <- raw_data[ids,]
   }
-
-  return(list("data"=raw_data, "trajectories"=trajectories))
+  
+  result <- c(list("data"=raw_data, "trajectories"=trajectories))
+  class(result) <- "mousetrap"
+  
+  return(result)
 }
