@@ -1,6 +1,7 @@
 #' Calculate sample entropy.
 #'
-#' Calculate sample entropy for each trajectory.
+#' Calculate sample entropy for each trajectory as a measure of the complexity
+#' of movements along one specific dimension.
 #' 
 #' \code{mt_sample_entropy} calculates the sample entropy for each trajectory as
 #' a measure of its complexity. Hehman et al (2015) provide details on how 
@@ -72,18 +73,16 @@
 #'   use="tn_trajectories", save_as="measures",
 #'   method="pracma", dimension="xpos", lag=3)
 #' 
+#' @author
+#' Pascal J. Kieslich (\email{kieslich@@psychologie.uni-mannheim.de})
+#' 
+#' Felix Henninger
+#' 
 #' @export
 mt_sample_entropy <- function(data,
   use="tn_trajectories", save_as="measures",
   dimension="xpos", method="pracma", lag=3, r=NULL,
-  verbose=FALSE,show_progress=NULL) {
-  
-  if(is.null(show_progress)==FALSE){
-    warning("The argument show_progress is deprecated. ",
-            "Please use verbose instead.",
-            call. = FALSE)
-    verbose <- show_progress
-  }
+  verbose=FALSE) {
   
   # Function to calculate sample entropy
   # based on Hehman et al. (2015)
@@ -131,20 +130,20 @@ mt_sample_entropy <- function(data,
   
   # Prepare data
   trajectories <- extract_data(data=data, use=use)
-  if (!(dimension %in% colnames(trajectories))) {
+  if (!(dimension %in% dimnames(trajectories)[[3]])) {
     stop(paste("Dimension", dimension, "not found in trajectory array."))
   }
   
   # Calculate number of logs
-  nlogs <- rowSums(!is.na(trajectories[,dimension,,drop=FALSE]))
+  nlogs <- mt_count(trajectories,dimensions=dimension)
   
   # Set r for sample entropy function
   # cf. Dale et al., 2007, p. 20
   if (is.null(r)) {
     if (length(nlogs) == 1) {
-      r <- .2 * stats::sd(diff(trajectories[,dimension,]), na.rm=TRUE)
+      r <- .2 * stats::sd(diff(trajectories[,,dimension]), na.rm=TRUE)
     } else {
-      r <- .2 * stats::sd(diff(t(trajectories[,dimension,])), na.rm=TRUE)
+      r <- .2 * stats::sd(diff(t(trajectories[,,dimension])), na.rm=TRUE)
     }
   }
     
@@ -169,7 +168,7 @@ mt_sample_entropy <- function(data,
   # Calculate measures
   for (i in 1:nrow(trajectories)){
     
-    current_values <- trajectories[i, dimension, 1:nlogs[i]]
+    current_values <- trajectories[i, 1:nlogs[i], dimension]
 
     # Calculate sample entropy
     
